@@ -11,19 +11,17 @@ from rasa.cli import SubParsersAction
 from rasa.cli.arguments import data as arguments
 from rasa.cli.arguments import default_arguments
 import rasa.cli.utils
-from rasa.core.training.converters.responses_prefix_converter import (
-    DomainResponsePrefixConverter,
-    StoryResponsePrefixConverter,
-)
-import rasa.nlu.convert
 from rasa.shared.constants import (
     DEFAULT_DATA_PATH,
     DEFAULT_CONFIG_PATH,
-    DEFAULT_DOMAIN_PATH,
     DOCS_URL_MIGRATION_GUIDE,
 )
 import rasa.shared.data
 from rasa.shared.core.constants import (
+    POLICY_NAME_FALLBACK,
+    POLICY_NAME_FORM,
+    POLICY_NAME_MAPPING,
+    POLICY_NAME_TWO_STAGE_FALLBACK,
     USER_INTENT_OUT_OF_SCOPE,
     ACTION_DEFAULT_FALLBACK_NAME,
 )
@@ -38,18 +36,13 @@ import rasa.shared.nlu.training_data.loading
 import rasa.shared.nlu.training_data.util
 import rasa.shared.utils.cli
 import rasa.utils.common
-from rasa.utils.converter import TrainingDataConverter
-from rasa.validator import Validator
 from rasa.shared.core.domain import Domain, InvalidDomain
 import rasa.shared.utils.io
-import rasa.core.config
-from rasa.core.policies.form_policy import FormPolicy
-from rasa.core.policies.fallback import FallbackPolicy
-from rasa.core.policies.two_stage_fallback import TwoStageFallbackPolicy
-from rasa.core.policies.mapping_policy import MappingPolicy
 
 if TYPE_CHECKING:
     from rasa.shared.core.training_data.structures import StoryStep
+    from rasa.validator import Validator
+    from rasa.utils.converter import TrainingDataConverter
 
 logger = logging.getLogger(__name__)
 
@@ -80,7 +73,7 @@ def add_subparser(
 
 
 def _add_data_convert_parsers(
-    data_subparsers, parents: List[argparse.ArgumentParser]
+    data_subparsers: SubParsersAction, parents: List[argparse.ArgumentParser]
 ) -> None:
     convert_parser = data_subparsers.add_parser(
         "convert",
@@ -157,7 +150,7 @@ def _add_data_convert_parsers(
 
 
 def _add_data_split_parsers(
-    data_subparsers, parents: List[argparse.ArgumentParser]
+    data_subparsers: SubParsersAction, parents: List[argparse.ArgumentParser]
 ) -> None:
     split_parser = data_subparsers.add_parser(
         "split",
@@ -181,7 +174,7 @@ def _add_data_split_parsers(
 
 
 def _add_data_validate_parsers(
-    data_subparsers, parents: List[argparse.ArgumentParser]
+    data_subparsers: SubParsersAction, parents: List[argparse.ArgumentParser]
 ) -> None:
     validate_parser = data_subparsers.add_parser(
         "validate",
@@ -242,6 +235,7 @@ def validate_files(args: argparse.Namespace, stories_only: bool = False) -> None
         args: Commandline arguments
         stories_only: If `True`, only the story structure is validated.
     """
+    from rasa.validator import Validator
 
     config = rasa.cli.utils.get_validated_path(
         args.config, "config", DEFAULT_CONFIG_PATH, none_is_valid=True
@@ -278,15 +272,15 @@ def validate_stories(args: argparse.Namespace) -> None:
     validate_files(args, stories_only=True)
 
 
-def _validate_domain(validator: Validator) -> bool:
+def _validate_domain(validator: "Validator") -> bool:
     return validator.verify_domain_validity()
 
 
-def _validate_nlu(validator: Validator, args: argparse.Namespace) -> bool:
+def _validate_nlu(validator: "Validator", args: argparse.Namespace) -> bool:
     return validator.verify_nlu(not args.fail_on_warnings)
 
 
-def _validate_story_structure(validator: Validator, args: argparse.Namespace) -> bool:
+def _validate_story_structure(validator: "Validator", args: argparse.Namespace) -> bool:
     # Check if a valid setting for `max_history` was given
     if isinstance(args.max_history, int) and args.max_history < 1:
         raise argparse.ArgumentTypeError(
@@ -299,6 +293,8 @@ def _validate_story_structure(validator: Validator, args: argparse.Namespace) ->
 
 
 def _convert_nlu_data(args: argparse.Namespace) -> None:
+    import rasa.nlu.convert
+
     from rasa.nlu.training_data.converters.nlu_markdown_to_yaml_converter import (
         NLUMarkdownToYamlConverter,
     )
@@ -356,8 +352,14 @@ def _convert_nlg_data(args: argparse.Namespace) -> None:
 
 def _migrate_responses(args: argparse.Namespace) -> None:
     """Migrate retrieval intent responses to the new 2.0 format.
+
     It does so modifying the stories and domain files.
     """
+    from rasa.core.training.converters.responses_prefix_converter import (
+        DomainResponsePrefixConverter,
+        StoryResponsePrefixConverter,
+    )
+
     if args.format == "yaml":
         rasa.utils.common.run_in_loop(
             _convert_to_yaml(args.out, args.domain, DomainResponsePrefixConverter())
@@ -374,7 +376,7 @@ def _migrate_responses(args: argparse.Namespace) -> None:
 
 
 async def _convert_to_yaml(
-    out_path: Text, data_path: Text, converter: TrainingDataConverter
+    out_path: Text, data_path: Text, converter: "TrainingDataConverter"
 ) -> None:
 
     output = Path(out_path)
@@ -415,7 +417,7 @@ async def _convert_to_yaml(
 
 
 async def _convert_file_to_yaml(
-    source_file: Path, target_dir: Path, converter: TrainingDataConverter
+    source_file: Path, target_dir: Path, converter: "TrainingDataConverter"
 ) -> bool:
     """Converts a single training data file to `YAML` format.
 
@@ -447,6 +449,8 @@ def _migrate_model_config(args: argparse.Namespace) -> None:
     Args:
         args: The commandline args with the required paths.
     """
+    import rasa.core.config
+
     configuration_file = Path(args.config)
     model_configuration = _get_configuration(configuration_file)
 
@@ -499,7 +503,7 @@ def _get_configuration(path: Path) -> Dict:
     _assert_two_stage_fallback_policy_is_migratable(config)
     _assert_only_one_fallback_policy_present(policy_names)
 
-    if FormPolicy.__name__ in policy_names:
+    if POLICY_NAME_FORM in policy_names:
         _warn_about_manual_forms_migration()
 
     return config
@@ -507,24 +511,24 @@ def _get_configuration(path: Path) -> Dict:
 
 def _assert_config_needs_migration(policies: List[Text]) -> None:
     migratable_policies = {
-        MappingPolicy.__name__,
-        FallbackPolicy.__name__,
-        TwoStageFallbackPolicy.__name__,
+        POLICY_NAME_MAPPING,
+        POLICY_NAME_FALLBACK,
+        POLICY_NAME_TWO_STAGE_FALLBACK,
     }
 
     if not migratable_policies.intersection((set(policies))):
         rasa.shared.utils.cli.print_error_and_exit(
             f"No policies were found which need migration. This command can migrate "
-            f"'{MappingPolicy.__name__}', '{FallbackPolicy.__name__}' and "
-            f"'{TwoStageFallbackPolicy.__name__}'."
+            f"'{POLICY_NAME_MAPPING}', '{POLICY_NAME_FALLBACK}' and "
+            f"'{POLICY_NAME_TWO_STAGE_FALLBACK}'."
         )
 
 
 def _warn_about_manual_forms_migration() -> None:
     rasa.shared.utils.cli.print_warning(
-        f"Your model configuration contains the '{FormPolicy.__name__}'. "
-        f"Note that this command does not migrate the '{FormPolicy.__name__}' and "
-        f"you have to migrate the '{FormPolicy.__name__}' manually. "
+        f"Your model configuration contains the '{POLICY_NAME_FORM}'. "
+        f"Note that this command does not migrate the '{POLICY_NAME_FORM}' and "
+        f"you have to migrate the '{POLICY_NAME_FORM}' manually. "
         f"Please see the migration guide for further details: "
         f"{DOCS_URL_MIGRATION_GUIDE}"
     )
@@ -533,7 +537,7 @@ def _warn_about_manual_forms_migration() -> None:
 def _assert_nlu_pipeline_given(config: Dict, policy_names: List[Text]) -> None:
     if not config.get("pipeline") and any(
         policy in policy_names
-        for policy in [FallbackPolicy.__name__, TwoStageFallbackPolicy.__name__]
+        for policy in [POLICY_NAME_FALLBACK, POLICY_NAME_TWO_STAGE_FALLBACK]
     ):
         rasa.shared.utils.cli.print_error_and_exit(
             "The model configuration has to include an NLU pipeline. This is required "
@@ -546,7 +550,7 @@ def _assert_two_stage_fallback_policy_is_migratable(config: Dict) -> None:
         (
             policy_config
             for policy_config in config.get("policies", [])
-            if policy_config.get("name") == TwoStageFallbackPolicy.__name__
+            if policy_config.get("name") == POLICY_NAME_TWO_STAGE_FALLBACK
         ),
         None,
     )
@@ -583,10 +587,7 @@ def _assert_two_stage_fallback_policy_is_migratable(config: Dict) -> None:
 
 
 def _assert_only_one_fallback_policy_present(policies: List[Text]) -> None:
-    if (
-        FallbackPolicy.__name__ in policies
-        and TwoStageFallbackPolicy.__name__ in policies
-    ):
+    if POLICY_NAME_FALLBACK in policies and POLICY_NAME_TWO_STAGE_FALLBACK in policies:
         rasa.shared.utils.cli.print_error_and_exit(
             "Your policy configuration contains two configured policies for handling "
             "fallbacks. Please decide on one."
