@@ -133,7 +133,7 @@ class HangoutsOutput(OutputChannel):
             new_messages = self._combine_cards(text_card, message)
 
         elif msg_new == "text":
-            new_messages = {"text": message.get("text")}
+            new_messages = {"text": message["text"]}
         else:
             new_messages = message
 
@@ -142,23 +142,21 @@ class HangoutsOutput(OutputChannel):
     async def send_text_message(
         self, recipient_id: Text, text: Text, **kwargs: Any
     ) -> None:
-
         await self._persist_message({"text": text})
 
-    async def send_image_url(self, recipient_id: Text, image: Text, **kwargs) -> None:
-
+    async def send_image_url(
+        self, recipient_id: Text, image: Text, **kwargs: Any
+    ) -> None:
         await self._persist_message(self._image_card(image))
 
     async def send_text_with_buttons(
-        self, recipient_id: Text, text: Text, buttons: List, **kwargs
+        self, recipient_id: Text, text: Text, buttons: List, **kwargs: Any
     ) -> None:
-
         await self._persist_message(self._text_button_card(text, buttons))
 
     async def send_attachment(
         self, recipient_id: Text, attachment: Text, **kwargs: Any
-    ):
-
+    ) -> None:
         await self.send_text_message(recipient_id, attachment)
 
     async def send_elements(
@@ -167,7 +165,7 @@ class HangoutsOutput(OutputChannel):
         raise NotImplementedError
 
     async def send_custom_json(
-        self, recipient_id: Text, json_message: Dict, **kwargs
+        self, recipient_id: Text, json_message: Dict, **kwargs: Any
     ) -> None:
         """Custom json payload is simply forwarded to Google Hangouts without
         any modifications. Use this for more complex cards, which can be created
