@@ -22,6 +22,7 @@ from rasa.nlu.constants import (
     RESPONSE_SELECTOR_DEFAULT_INTENT,
     RESPONSE_SELECTOR_RETRIEVAL_INTENTS,
     TOKENS_NAMES,
+    ENTITY_ATTRIBUTE_CONFIDENCE,
 )
 from rasa.shared.nlu.constants import (
     INTENT,
@@ -37,6 +38,8 @@ from rasa.shared.nlu.constants import (
     RESPONSE_SELECTOR,
     FULL_RETRIEVAL_INTENT_NAME_KEY,
     TEXT,
+    ENTITY_ATTRIBUTE_GROUP,
+    ENTITY_ATTRIBUTE_ROLE,
 )
 from rasa.constants import RESULTS_FILE, PERCENTAGE_KEY
 from rasa.shared.core.events import (
@@ -54,6 +57,23 @@ if typing.TYPE_CHECKING:
     from rasa.core.processor import MessageProcessor
     from rasa.shared.core.generator import TrainingDataGenerator
 
+    from typing_extensions import TypedDict
+
+    EntityPrediction = TypedDict(
+        "EntityPrediction",
+        {
+            ENTITY_ATTRIBUTE_TEXT: Text,
+            ENTITY_ATTRIBUTE_START: Optional[float],
+            ENTITY_ATTRIBUTE_END: Optional[float],
+            ENTITY_ATTRIBUTE_VALUE: Text,
+            ENTITY_ATTRIBUTE_CONFIDENCE: float,
+            ENTITY_ATTRIBUTE_TYPE: Text,
+            ENTITY_ATTRIBUTE_GROUP: Optional[Text],
+            ENTITY_ATTRIBUTE_ROLE: Optional[Text],
+            "additional_info": Any,
+        },
+        total=False,
+    )
 
 CONFUSION_MATRIX_STORIES_FILE = "story_confusion_matrix.png"
 REPORT_STORIES_FILE = "story_report.json"
@@ -74,6 +94,8 @@ StoryEvaluation = namedtuple(
     ],
 )
 
+PredictionList = List[Optional[Text]]
+
 
 class WrongPredictionException(RasaException, ValueError):
     """Raised if a wrong prediction is encountered."""
@@ -84,31 +106,31 @@ class EvaluationStore:
 
     def __init__(
         self,
-        action_predictions: Optional[List[Text]] = None,
-        action_targets: Optional[List[Text]] = None,
-        intent_predictions: Optional[List[Text]] = None,
-        intent_targets: Optional[List[Text]] = None,
-        entity_predictions: Optional[List[Dict[Text, Any]]] = None,
-        entity_targets: Optional[List[Dict[Text, Any]]] = None,
+        action_predictions: Optional[PredictionList] = None,
+        action_targets: Optional[PredictionList] = None,
+        intent_predictions: Optional[PredictionList] = None,
+        intent_targets: Optional[PredictionList] = None,
+        entity_predictions: Optional[List["EntityPrediction"]] = None,
+        entity_targets: Optional[List["EntityPrediction"]] = None,
     ) -> None:
+        """Initialize store attributes."""
         self.action_predictions = action_predictions or []
         self.action_targets = action_targets or []
         self.intent_predictions = intent_predictions or []
         self.intent_targets = intent_targets or []
-        self.entity_predictions = entity_predictions or []
-        self.entity_targets = entity_targets or []
+        self.entity_predictions: List["EntityPrediction"] = entity_predictions or []
+        self.entity_targets: List["EntityPrediction"] = entity_targets or []
 
     def add_to_store(
         self,
-        action_predictions: Optional[List[Text]] = None,
-        action_targets: Optional[List[Text]] = None,
-        intent_predictions: Optional[List[Text]] = None,
-        intent_targets: Optional[List[Text]] = None,
-        entity_predictions: Optional[List[Dict[Text, Any]]] = None,
-        entity_targets: Optional[List[Dict[Text, Any]]] = None,
+        action_predictions: Optional[PredictionList] = None,
+        action_targets: Optional[PredictionList] = None,
+        intent_predictions: Optional[PredictionList] = None,
+        intent_targets: Optional[PredictionList] = None,
+        entity_predictions: Optional[List["EntityPrediction"]] = None,
+        entity_targets: Optional[List["EntityPrediction"]] = None,
     ) -> None:
-        """Add items or lists of items to the store"""
-
+        """Add items or lists of items to the store."""
         self.action_predictions.extend(action_predictions or [])
         self.action_targets.extend(action_targets or [])
         self.intent_targets.extend(intent_targets or [])
@@ -117,7 +139,7 @@ class EvaluationStore:
         self.entity_targets.extend(entity_targets or [])
 
     def merge_store(self, other: "EvaluationStore") -> None:
-        """Add the contents of other to self"""
+        """Add the contents of other to self."""
         self.add_to_store(
             action_predictions=other.action_predictions,
             action_targets=other.action_targets,
@@ -136,8 +158,8 @@ class EvaluationStore:
 
     @staticmethod
     def _compare_entities(
-        entity_predictions: List[Dict[Text, Any]],
-        entity_targets: List[Dict[Text, Any]],
+        entity_predictions: List["EntityPrediction"],
+        entity_targets: List["EntityPrediction"],
         i_pred: int,
         i_target: int,
     ) -> int:
@@ -155,16 +177,16 @@ class EvaluationStore:
             target = entity_targets[i_target]
         if target and pred:
             # Check which entity has the lower "start" value
-            if pred.get("start") < target.get("start"):
+            if pred.get(ENTITY_ATTRIBUTE_START) < target.get(ENTITY_ATTRIBUTE_START):
                 return -1
-            elif target.get("start") < pred.get("start"):
+            elif target.get(ENTITY_ATTRIBUTE_START) < pred.get(ENTITY_ATTRIBUTE_START):
                 return 1
             else:
                 # Since both have the same "start" values,
                 # check which one has the lower "end" value
-                if pred.get("end") < target.get("end"):
+                if pred.get(ENTITY_ATTRIBUTE_END) < target.get(ENTITY_ATTRIBUTE_END):
                     return -1
-                elif target.get("end") < pred.get("end"):
+                elif target.get(ENTITY_ATTRIBUTE_END) < pred.get(ENTITY_ATTRIBUTE_END):
                     return 1
                 else:
                     # The entities have the same "start" and "end" values
@@ -175,13 +197,13 @@ class EvaluationStore:
     def _generate_entity_training_data(entity: Dict[Text, Any]) -> Text:
         return TrainingDataWriter.generate_entity(entity.get("text"), entity)
 
-    def serialise(self) -> Tuple[List[Text], List[Text]]:
+    def serialise(self) -> Tuple[PredictionList, PredictionList]:
         """Turn targets and predictions to lists of equal size for sklearn."""
         texts = sorted(
             list(
                 set(
-                    [e.get("text") for e in self.entity_targets]
-                    + [e.get("text") for e in self.entity_predictions]
+                    [e.get("text", "") for e in self.entity_targets]
+                    + [e.get("text", "") for e in self.entity_predictions]
                 )
             )
         )
@@ -192,12 +214,17 @@ class EvaluationStore:
         for text in texts:
             # sort the entities of this sentence to compare them directly
             entity_targets = sorted(
-                filter(lambda x: x.get("text") == text, self.entity_targets),
-                key=lambda x: x.get("start"),
+                filter(
+                    lambda x: x.get(ENTITY_ATTRIBUTE_TEXT) == text, self.entity_targets
+                ),
+                key=lambda x: x.get(ENTITY_ATTRIBUTE_START),
             )
             entity_predictions = sorted(
-                filter(lambda x: x.get("text") == text, self.entity_predictions),
-                key=lambda x: x.get("start"),
+                filter(
+                    lambda x: x.get(ENTITY_ATTRIBUTE_TEXT) == text,
+                    self.entity_predictions,
+                ),
+                key=lambda x: x.get(ENTITY_ATTRIBUTE_START),
             )
 
             i_pred, i_target = 0, 0
@@ -297,11 +324,12 @@ class WronglyClassifiedUserUtterance(UserUttered):
     type_name = "wrong_utterance"
 
     def __init__(self, event: UserUttered, eval_store: EvaluationStore) -> None:
-
-        if not eval_store.intent_predictions:
-            self.predicted_intent = None
-        else:
+        """Set `predicted_intent` and `predicted_entities` attributes."""
+        try:
             self.predicted_intent = eval_store.intent_predictions[0]
+        except LookupError:
+            self.predicted_intent = None
+
         self.predicted_entities = eval_store.entity_predictions
 
         intent = {"name": eval_store.intent_targets[0]}
@@ -341,14 +369,18 @@ async def _create_data_generator(
     resource_name: Text,
     agent: "Agent",
     max_stories: Optional[int] = None,
-    use_e2e: bool = False,
+    use_conversation_test_files: bool = False,
 ) -> "TrainingDataGenerator":
     from rasa.shared.core.generator import TrainingDataGenerator
 
     test_data_importer = TrainingDataImporter.load_from_dict(
         training_data_paths=[resource_name]
     )
-    story_graph = await test_data_importer.get_stories(use_e2e=use_e2e)
+    if use_conversation_test_files:
+        story_graph = await test_data_importer.get_conversation_tests()
+    else:
+        story_graph = await test_data_importer.get_stories()
+
     return TrainingDataGenerator(
         story_graph,
         agent.domain,
@@ -360,7 +392,7 @@ async def _create_data_generator(
 
 def _clean_entity_results(
     text: Text, entity_results: List[Dict[Text, Any]]
-) -> List[Dict[Text, Any]]:
+) -> List["EntityPrediction"]:
     """Extract only the token variables from an entity dict."""
     cleaned_entities = []
 
@@ -444,7 +476,7 @@ def _collect_user_uttered_predictions(
         predicted_base_intent = _get_full_retrieval_intent(predicted)
 
     user_uttered_eval_store.add_to_store(
-        intent_predictions=[predicted_base_intent], intent_targets=[intent_gold]
+        intent_targets=[intent_gold], intent_predictions=[predicted_base_intent]
     )
 
     entity_gold = event.entities
@@ -828,7 +860,7 @@ async def test(
     Returns:
         Evaluation summary.
     """
-    from rasa.test import get_evaluation_metrics
+    from rasa.model_testing import get_evaluation_metrics
 
     generator = await _create_data_generator(stories, agent, max_stories, e2e)
     completed_trackers = generator.generate_story_trackers()
@@ -946,7 +978,9 @@ def _log_evaluation_table(
 
 
 def _plot_story_evaluation(
-    targets: List[Text], predictions: List[Text], output_directory: Optional[Text]
+    targets: PredictionList,
+    predictions: PredictionList,
+    output_directory: Optional[Text],
 ) -> None:
     """Plot a confusion matrix of story evaluation."""
     from sklearn.metrics import confusion_matrix
@@ -970,14 +1004,19 @@ def _plot_story_evaluation(
 
 
 async def compare_models_in_dir(
-    model_dir: Text, stories_file: Text, output: Text
+    model_dir: Text,
+    stories_file: Text,
+    output: Text,
+    use_conversation_test_files: bool = False,
 ) -> None:
-    """Evaluate multiple trained models in a directory on a test set.
+    """Evaluates multiple trained models in a directory on a test set.
 
     Args:
         model_dir: path to directory that contains the models to evaluate
         stories_file: path to the story file
         output: output directory to store results to
+        use_conversation_test_files: `True` if conversation test files should be used
+            for testing instead of regular Core story files.
     """
     number_correct = defaultdict(list)
 
@@ -991,7 +1030,11 @@ async def compare_models_in_dir(
             # The model files are named like <config-name>PERCENTAGE_KEY<number>.tar.gz
             # Remove the percentage key and number from the name to get the config name
             config_name = os.path.basename(model).split(PERCENTAGE_KEY)[0]
-            number_of_correct_stories = await _evaluate_core_model(model, stories_file)
+            number_of_correct_stories = await _evaluate_core_model(
+                model,
+                stories_file,
+                use_conversation_test_files=use_conversation_test_files,
+            )
             number_correct_in_run[config_name].append(number_of_correct_stories)
 
         for k, v in number_correct_in_run.items():
@@ -1002,18 +1045,27 @@ async def compare_models_in_dir(
     )
 
 
-async def compare_models(models: List[Text], stories_file: Text, output: Text) -> None:
-    """Evaluate provided trained models on a test set.
+async def compare_models(
+    models: List[Text],
+    stories_file: Text,
+    output: Text,
+    use_conversation_test_files: bool = False,
+) -> None:
+    """Evaluates multiple trained models on a test set.
 
     Args:
-        models: list of trained model paths
+        models: Paths to model files.
         stories_file: path to the story file
         output: output directory to store results to
+        use_conversation_test_files: `True` if conversation test files should be used
+            for testing instead of regular Core story files.
     """
     number_correct = defaultdict(list)
 
     for model in models:
-        number_of_correct_stories = await _evaluate_core_model(model, stories_file)
+        number_of_correct_stories = await _evaluate_core_model(
+            model, stories_file, use_conversation_test_files=use_conversation_test_files
+        )
         number_correct[os.path.basename(model)].append(number_of_correct_stories)
 
     rasa.shared.utils.io.dump_obj_as_json_to_file(
@@ -1021,13 +1073,17 @@ async def compare_models(models: List[Text], stories_file: Text, output: Text) -
     )
 
 
-async def _evaluate_core_model(model: Text, stories_file: Text) -> int:
+async def _evaluate_core_model(
+    model: Text, stories_file: Text, use_conversation_test_files: bool = False
+) -> int:
     from rasa.core.agent import Agent
 
     logger.info(f"Evaluating model '{model}'")
 
     agent = Agent.load(model)
-    generator = await _create_data_generator(stories_file, agent)
+    generator = await _create_data_generator(
+        stories_file, agent, use_conversation_test_files=use_conversation_test_files
+    )
     completed_trackers = generator.generate_story_trackers()
 
     # Entities are ignored here as we only compare number of correct stories.
