@@ -113,12 +113,12 @@ class RegexEntityExtractor(EntityExtractor):
     def load(
         cls,
         meta: Dict[Text, Any],
-        model_dir: Optional[Text] = None,
+        model_dir: Text,
         model_metadata: Optional[Metadata] = None,
         cached_component: Optional["RegexEntityExtractor"] = None,
         **kwargs: Any,
     ) -> "RegexEntityExtractor":
-
+        """Loads trained component (see parent class for full docstring)."""
         file_name = meta.get("file")
         regex_file = os.path.join(model_dir, file_name)
 
